@@ -14,22 +14,18 @@
 
 //! Cloud API.
 
+use super::Result;
+use super::auth::{self, AuthMethod};
 #[allow(unused_imports)]
-use fallible_iterator::FallibleIterator;
-
-#[allow(unused_imports)]
-use super::{Error, ErrorKind, Result};
-use super::auth::AuthMethod;
+use super::common::FlavorRef;
 #[cfg(feature = "compute")]
-use super::compute::{Flavor, FlavorQuery, FlavorSummary,
-                     Server, ServerQuery, ServerSummary};
+use super::compute::{Flavor, FlavorQuery, FlavorSummary, KeyPair, KeyPairQuery,
+                     NewServer, Server, ServerQuery, ServerSummary};
 #[cfg(feature = "image")]
 use super::image::{Image, ImageQuery};
 #[cfg(feature = "network")]
 use super::network::{Network, NetworkQuery};
 use super::session::Session;
-#[allow(unused_imports)]
-use super::utils::ResultExt;
 
 
 /// OpenStack cloud API.
@@ -43,24 +39,55 @@ pub struct Cloud {
 impl Cloud {
     /// Create a new cloud object with a given authentication plugin.
     ///
-    /// See (auth module)[auth/index.html) for details on how to authenticate
+    /// See [`auth` module](auth/index.html) for details on how to authenticate
     /// against OpenStack clouds.
     ///
     /// # Example
     ///
     /// ```rust,no_run
-    /// use openstack;
+    /// fn cloud_from_env() -> openstack::Result<openstack::Cloud> {
+    ///     openstack::auth::from_env().map(openstack::Cloud::new)
+    /// }
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// # fn main() { cloud_from_env().unwrap(); }
     /// ```
+    ///
+    /// Note: in this particular case it's better to use
+    /// [from_env](#method.from_env).
     pub fn new<Auth: AuthMethod + 'static>(auth_method: Auth) -> Cloud {
         Cloud {
             session: Session::new(auth_method)
         }
     }
 
+    /// Create a new cloud object from environment variables.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// # fn cloud_from_env() -> openstack::Result<()> {
+    /// let os = openstack::Cloud::from_env()?;
+    /// # Ok(()) }
+    /// # fn main() { cloud_from_env().unwrap(); }
+    /// ```
+    pub fn from_env() -> Result<Cloud> {
+        Ok(Cloud {
+            session: Session::new(auth::from_env()?)
+        })
+    }
+
     /// Convert this cloud into one using the given endpoint interface.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// fn cloud_from_env() -> openstack::Result<openstack::Cloud> {
+    ///     openstack::Cloud::from_env()
+    ///         .map(|os| os.with_endpoint_interface("internal"))
+    /// }
+    ///
+    /// # fn main() { cloud_from_env().unwrap(); }
+    /// ```
     pub fn with_endpoint_interface<S>(self, endpoint_interface: S)
             -> Cloud where S: Into<String> {
         Cloud {
@@ -96,6 +123,15 @@ impl Cloud {
         ImageQuery::new(&self.session)
     }
 
+    /// Build a query against key pairs list.
+    ///
+    /// The returned object is a builder that should be used to construct
+    /// the query.
+    #[cfg(feature = "compute")]
+    pub fn find_keypairs(&self) -> KeyPairQuery {
+        KeyPairQuery::new(&self.session)
+    }
+
     /// Build a query against network list.
     ///
     /// The returned object is a builder that should be used to construct
@@ -117,8 +153,7 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let sorting = openstack::compute::ServerSortKey::AccessIpv4;
     /// let server_list = os.find_servers()
     ///     .sort_by(openstack::Sort::Asc(sorting)).with_limit(5)
@@ -136,30 +171,12 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let server = os.get_flavor("m1.medium").expect("Unable to get a flavor");
     /// ```
     #[cfg(feature = "compute")]
-    pub fn get_flavor<Id: Into<String>>(&self, id_or_name: Id) -> Result<Flavor> {
-        let s = id_or_name.into();
-        Flavor::new(&self.session, &s).if_not_found_then(|| {
-            self.find_flavors().into_iter()
-                .filter(|item| item.name() == &s).take(2)
-                .collect::<Vec<FlavorSummary>>().and_then(|mut items| {
-                    if items.len() > 1 {
-                        Err(Error::new(ErrorKind::TooManyItems,
-                                       "Too many flavors with this name"))
-                    } else {
-                        match items.pop() {
-                            Some(item) => item.details(),
-                            None => Err(Error::new(
-                                ErrorKind::ResourceNotFound,
-                                "No flavors with this name or ID"))
-                        }
-                    }
-                })
-        })
+    pub fn get_flavor<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Flavor> {
+        Flavor::new(&self.session, id_or_name)
     }
 
     /// Find an image by its name or ID.
@@ -169,16 +186,27 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let server = os.get_image("centos7").expect("Unable to get a image");
     /// ```
     #[cfg(feature = "image")]
-    pub fn get_image<Id: Into<String>>(&self, id_or_name: Id) -> Result<Image> {
-        let s = id_or_name.into();
-        Image::new(&self.session, &s).if_not_found_then(|| {
-            self.find_images().with_name(s).one()
-        })
+    pub fn get_image<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Image> {
+        Image::new(&self.session, id_or_name)
+    }
+
+    /// Find a key pair by its name or ID.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let server = os.get_keypair("default").expect("Unable to get a key pair");
+    /// ```
+    #[cfg(feature = "compute")]
+    pub fn get_keypair<Id: AsRef<str>>(&self, name: Id) -> Result<KeyPair> {
+        KeyPair::new(&self.session, name)
     }
 
     /// Find an network by its name or ID.
@@ -188,16 +216,12 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let server = os.get_network("centos7").expect("Unable to get a network");
     /// ```
     #[cfg(feature = "network")]
-    pub fn get_network<Id: Into<String>>(&self, id_or_name: Id) -> Result<Network> {
-        let s = id_or_name.into();
-        Network::new(&self.session, &s).if_not_found_then(|| {
-            self.find_networks().with_name(s).one()
-        })
+    pub fn get_network<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Network> {
+        Network::new(&self.session, id_or_name)
     }
 
     /// Find a server by its name or ID.
@@ -207,31 +231,13 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let server = os.get_server("8a1c355b-2e1e-440a-8aa8-f272df72bc32")
     ///     .expect("Unable to get a server");
     /// ```
     #[cfg(feature = "compute")]
-    pub fn get_server<Id: Into<String>>(&self, id_or_name: Id) -> Result<Server> {
-        let s = id_or_name.into();
-        Server::new(&self.session, &s).if_not_found_then(|| {
-            self.find_servers().with_name(s.clone()).into_iter()
-                .filter(|srv| srv.name() == &s).take(2)
-                .collect::<Vec<ServerSummary>>().and_then(|mut srvs| {
-                    if srvs.len() > 1 {
-                        Err(Error::new(ErrorKind::TooManyItems,
-                                       "Too many servers with this name"))
-                    } else {
-                        match srvs.pop() {
-                            Some(srv) => srv.details(),
-                            None => Err(Error::new(
-                                ErrorKind::ResourceNotFound,
-                                "No servers with this name or ID"))
-                        }
-                    }
-                })
-        })
+    pub fn get_server<Id: AsRef<str>>(&self, id_or_name: Id) -> Result<Server> {
+        Server::new(&self.session, id_or_name)
     }
 
     /// List all flavors.
@@ -245,8 +251,7 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let server_list = os.list_flavors().expect("Unable to fetch flavors");
     /// ```
     #[cfg(feature = "compute")]
@@ -265,8 +270,7 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let server_list = os.list_images().expect("Unable to fetch images");
     /// ```
     #[cfg(feature = "image")]
@@ -274,6 +278,21 @@ impl Cloud {
         self.find_images().all()
     }
 
+    /// List all key pairs.
+    ///
+    /// # Example
+    ///
+    /// ```rust,no_run
+    /// use openstack;
+    ///
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
+    /// let result = os.list_keypairs().expect("Unable to fetch key pairs");
+    /// ```
+    #[cfg(feature = "compute")]
+    pub fn list_keypairs(&self) -> Result<Vec<KeyPair>> {
+        self.find_keypairs().all()
+    }
+
     /// List all networks.
     ///
     /// This call can yield a lot of results, use the
@@ -285,8 +304,7 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let server_list = os.list_networks().expect("Unable to fetch networks");
     /// ```
     #[cfg(feature = "network")]
@@ -305,14 +323,23 @@ impl Cloud {
     /// ```rust,no_run
     /// use openstack;
     ///
-    /// let auth = openstack::auth::from_env().expect("Unable to authenticate");
-    /// let os = openstack::Cloud::new(auth);
+    /// let os = openstack::Cloud::from_env().expect("Unable to authenticate");
     /// let server_list = os.list_servers().expect("Unable to fetch servers");
     /// ```
     #[cfg(feature = "compute")]
     pub fn list_servers(&self) -> Result<Vec<ServerSummary>> {
         self.find_servers().all()
     }
+
+    /// Prepare a new server for creation.
+    ///
+    /// This call returns a `NewServer` object, which is a builder to populate
+    /// server fields.
+    #[cfg(feature = "compute")]
+    pub fn new_server<S, F>(&self, name: S, flavor: F) -> NewServer
+            where S: Into<String>, F: Into<FlavorRef> {
+        NewServer::new(&self.session, name.into(), flavor.into())
+    }
 }
 
 
