@@ -43,59 +43,108 @@ pub trait ResourceId {
 }
 
 macro_rules! opaque_resource_type {
-    ($(#[$attr:meta])* $name:ident) => (
+    ($(#[$attr:meta])* $name:ident ? $service:expr) => (
         $(#[$attr])*
-        #[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
-        pub struct $name(String);
+        #[derive(Debug, Clone, PartialEq, Eq)]
+        pub struct $name {
+            pub(crate) value: String,
+            pub(crate) verified: bool
+        }
 
         impl From<String> for $name {
             fn from(value: String) -> $name {
-                $name(value)
+                $name {
+                    value: value,
+                    verified: false
+                }
             }
         }
 
         impl<'s> From<&'s str> for $name {
             fn from(value: &'s str) -> $name {
-                $name(String::from(value))
+                $name {
+                    value: String::from(value),
+                    verified: false
+                }
             }
         }
 
         impl From<$name> for String {
             fn from(value: $name) -> String {
-                value.0
+                value.value
             }
         }
 
         impl AsRef<str> for $name {
             fn as_ref(&self) -> &str {
-                &self.0
+                &self.value
             }
         }
 
         impl ::std::fmt::Display for $name {
             fn fmt(&self, f: &mut ::std::fmt::Formatter) -> ::std::fmt::Result {
-                self.0.fmt(f)
+                self.value.fmt(f)
+            }
+        }
+
+        impl ::serde::ser::Serialize for $name {
+            fn serialize<S>(&self, serializer: S) -> ::std::result::Result<S::Ok, S::Error>
+                    where S: ::serde::ser::Serializer {
+                serializer.serialize_str(&self.value)
+            }
+        }
+
+        impl<'de> ::serde::de::Deserialize<'de> for $name {
+            fn deserialize<D>(deserializer: D)
+                    -> ::std::result::Result<$name, D::Error>
+                    where D: ::serde::de::Deserializer<'de> {
+                Ok($name {
+                    value: String::deserialize(deserializer)?,
+                    // Assume that values coming from network are valid.
+                    verified: true
+                })
+            }
+        }
+
+        impl $name {
+            /// Create a reference that was previously verified.
+            #[allow(dead_code)]
+            pub(crate) fn new_verified(value: String) -> $name {
+                $name {
+                    value: value,
+                    verified: true
+                }
+            }
+
+            /// Verify this reference and convert to an ID, if possible.
+            #[cfg(not(feature = $service))]
+            #[allow(dead_code)]
+            pub(crate) fn into_verified(self, _session: &$crate::session::Session)
+                    -> $crate::Result<String> {
+                Ok(self.value)
             }
         }
     )
 }
 
-opaque_resource_type!(#[doc = "An ID of a `Flavor`"] FlavorRef);
+opaque_resource_type!(#[doc = "An ID of a `Flavor`"] FlavorRef ? "compute");
 
-opaque_resource_type!(#[doc = "An ID of an `Image`"] ImageRef);
+opaque_resource_type!(#[doc = "An ID of an `Image`"] ImageRef ? "image");
 
-opaque_resource_type!(#[doc = "An ID of a `Network`"] NetworkRef);
+opaque_resource_type!(#[doc = "An ID of a `Network`"] NetworkRef ? "network");
 
-opaque_resource_type!(#[doc = "An ID of a `Project`"] ProjectRef);
+opaque_resource_type!(#[doc = "An ID of a `Project`"] ProjectRef ? "identity");
 
-opaque_resource_type!(#[doc = "An ID of a `User`"] UserRef);
+opaque_resource_type!(#[doc = "An ID of a `Port`"] PortRef ? "network");
+
+opaque_resource_type!(#[doc = "An ID of a `User`"] UserRef ? "identity");
 
 
 #[cfg(test)]
 mod test {
     use serde_json;
 
-    opaque_resource_type!(TestId);
+    opaque_resource_type!(TestId ? "test");
 
     #[test]
     fn test_opaque_type_basics() {
