@@ -261,12 +261,24 @@ impl<'session> IntoFallibleIterator for FlavorQuery<'session> {
 
 impl<'session> From<Flavor<'session>> for FlavorRef {
     fn from(value: Flavor<'session>) -> FlavorRef {
-        FlavorRef::from(value.inner.id)
+        FlavorRef::new_verified(value.inner.id)
     }
 }
 
 impl<'session> From<FlavorSummary<'session>> for FlavorRef {
     fn from(value: FlavorSummary<'session>) -> FlavorRef {
-        FlavorRef::from(value.inner.id)
+        FlavorRef::new_verified(value.inner.id)
+    }
+}
+
+impl FlavorRef {
+    /// Verify this reference and convert to an ID, if possible.
+    #[cfg(feature = "compute")]
+    pub(crate) fn into_verified(self, session: &Session) -> Result<String> {
+        Ok(if self.verified {
+            self.value
+        } else {
+            session.get_flavor(&self.value)?.id
+        })
     }
 }
