@@ -113,6 +113,14 @@ protocol_enum! {
     }
 }
 
+protocol_enum! {
+    #[doc = "Type of a key pair."]
+    enum KeyPairType {
+        SSH = "ssh",
+        X509 = "x509"
+    }
+}
+
 /// Address of a server.
 #[derive(Clone, Debug, Deserialize)]
 pub struct ServerAddress {
@@ -165,6 +173,35 @@ pub struct ServerRoot {
     pub server: Server
 }
 
+#[derive(Clone, Debug, Serialize)]
+#[serde(untagged)]
+pub enum ServerNetwork {
+    Network { uuid: String },
+    Port { port: String },
+    FixedIp { fixed_ip: Ipv4Addr }
+}
+
+#[derive(Clone, Debug, Serialize)]
+pub struct ServerCreate {
+    pub flavorRef: String,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub imageRef: Option<String>,
+    #[serde(skip_serializing_if = "Option::is_none")]
+    pub key_name: Option<String>,
+    pub name: String,
+    pub networks: Vec<ServerNetwork>
+}
+
+#[derive(Clone, Debug, Serialize)]
+pub struct ServerCreateRoot {
+    pub server: ServerCreate
+}
+
+#[derive(Clone, Debug, Deserialize)]
+pub struct CreatedServerRoot {
+    pub server: common::protocol::Ref
+}
+
 #[derive(Clone, Debug, Deserialize)]
 pub struct Flavor {
     #[serde(rename = "OS-FLV-EXT-DATA:ephemeral", default)]
@@ -197,6 +234,25 @@ pub struct FlavorRoot {
     pub flavor: Flavor
 }
 
+#[derive(Clone, Debug, Deserialize)]
+pub struct KeyPair {
+    pub fingerprint: String,
+    #[serde(rename = "type", default)]
+    pub key_type: Option<KeyPairType>,
+    pub name: String,
+    pub public_key: String,
+}
+
+#[derive(Clone, Debug, Deserialize)]
+pub struct KeyPairsRoot {
+    pub keypairs: Vec<KeyPair>
+}
+
+#[derive(Clone, Debug, Deserialize)]
+pub struct KeyPairRoot {
+    pub keypair: KeyPair
+}
+
 impl Default for ServerStatus {
     fn default() -> ServerStatus {
         ServerStatus::Unknown
