@@ -22,11 +22,12 @@ use std::time::Duration;
 use chrono::{DateTime, FixedOffset};
 use fallible_iterator::{IntoFallibleIterator, FallibleIterator};
 use serde::Serialize;
+use waiter::{Waiter, WaiterCurrentState};
 
-use super::super::{Error, ErrorKind, Result, Sort, Waiter};
+use super::super::{Error, ErrorKind, Result, Sort};
 use super::super::common::{self, DeletionWaiter, FlavorRef, ImageRef,
-                           ListResources, ProjectRef, Refresh, ResourceId,
-                           ResourceIterator, UserRef};
+                           ListResources, NetworkRef, PortRef, ProjectRef,
+                           Refresh, ResourceId, ResourceIterator, UserRef};
 use super::super::session::Session;
 use super::super::utils::Query;
 use super::base::V2API;
@@ -62,6 +63,33 @@ pub struct ServerStatusWaiter<'server> {
     target: protocol::ServerStatus
 }
 
+/// A virtual NIC of a new server.
+#[derive(Clone, Debug)]
+pub enum ServerNIC {
+    /// A NIC from the given network.
+    FromNetwork(NetworkRef),
+    /// A NIC with the given port.
+    WithPort(PortRef),
+    /// A NIC with the given fixed IP.
+    WithFixedIp(Ipv4Addr)
+}
+
+/// A request to create a server.
+#[derive(Debug)]
+pub struct NewServer<'session> {
+    session: &'session Session,
+    flavor: FlavorRef,
+    image: Option<ImageRef>,
+    name: String,
+    networks: Vec<ServerNIC>,
+}
+
+/// Waiter for server to be created.
+#[derive(Debug)]
+pub struct ServerCreationWaiter<'server> {
+    server: Server<'server>
+}
+
 
 impl<'session> Refresh for Server<'session> {
     /// Refresh the server.
@@ -201,16 +229,7 @@ impl<'session> Server<'session> {
     }
 }
 
-impl<'server> ServerStatusWaiter<'server> {
-    /// Current state of the server.
-    ///
-    /// Valid as of the last poll.
-    pub fn current(&self) -> &Server<'server> {
-        self.server
-    }
-}
-
-impl<'server> Waiter<()> for ServerStatusWaiter<'server> {
+impl<'server> Waiter<(), Error> for ServerStatusWaiter<'server> {
     fn default_wait_timeout(&self) -> Option<Duration> {
         // TODO(dtantsur): vary depending on target?
         Some(Duration::new(600, 0))
@@ -220,9 +239,10 @@ impl<'server> Waiter<()> for ServerStatusWaiter<'server> {
         Duration::new(1, 0)
     }
 
-    fn timeout_error_message(&self) -> String {
-        format!("Timeout waiting for server {} to reach state {}",
-                self.server.id(), self.target)
+    fn timeout_error(&self) -> Error {
+        Error::new(ErrorKind::OperationTimedOut,
+                   format!("Timeout waiting for server {} to reach state {}",
+                           self.server.id(), self.target))
     }
 
     fn poll(&mut self) -> Result<Option<()>> {
@@ -244,6 +264,12 @@ impl<'server> Waiter<()> for ServerStatusWaiter<'server> {
     }
 }
 
+impl<'server> WaiterCurrentState<Server<'server>> for ServerStatusWaiter<'server> {
+    fn waiter_current_state(&self) -> &Server<'server> {
+        &self.server
+    }
+}
+
 impl<'session> ServerSummary<'session> {
     /// Get a reference to server unique ID.
     pub fn id(&self) -> &String {
@@ -425,6 +451,155 @@ impl<'session> ServerQuery<'session> {
     }
 }
 
+fn convert_networks(session: &Session, networks: Vec<ServerNIC>)
+        -> Result<Vec<protocol::ServerNetwork>> {
+    let mut result = Vec::with_capacity(networks.len());
+    for item in networks {
+        result.push(match item {
+            ServerNIC::FromNetwork(n) => protocol::ServerNetwork::Network {
+                uuid: n.into_verified(session)?
+            },
+            ServerNIC::WithPort(p) =>
+                protocol::ServerNetwork::Port { port: p.into() },
+            ServerNIC::WithFixedIp(ip) =>
+                protocol::ServerNetwork::FixedIp{ fixed_ip: ip }
+        });
+    }
+    Ok(result)
+}
+
+impl<'session> NewServer<'session> {
+    /// Start creating a server.
+    pub(crate) fn new(session: &'session Session, name: String, flavor: FlavorRef)
+            -> NewServer<'session> {
+        NewServer {
+            session: session,
+            flavor: flavor,
+            image: None,
+            name: name,
+            networks: Vec::new(),
+        }
+    }
+
+    /// Request creation of the server.
+    pub fn create(self) -> Result<ServerCreationWaiter<'session>> {
+        let request = protocol::ServerCreate {
+            flavorRef: self.flavor.into_verified(self.session)?,
+            imageRef: match self.image {
+                Some(img) => Some(img.into_verified(self.session)?),
+                None => None
+            },
+            key_name: None,  // TODO
+            name: self.name,
+            networks: convert_networks(self.session, self.networks)?
+        };
+
+        let server_ref = self.session.create_server(request)?;
+        Ok(ServerCreationWaiter {
+            server: Server::new(self.session, server_ref.id)?
+        })
+    }
+
+    /// Add a virtual NIC with given fixed IP to the new server.
+    ///
+    /// A shorthand for `add_nic`.
+    pub fn add_fixed_ip(&mut self, fixed_ip: Ipv4Addr) {
+        self.add_nic(ServerNIC::WithFixedIp(fixed_ip));
+    }
+
+    /// Add a virtual NIC from this network to the new server.
+    ///
+    /// A shorthand for `add_nic`.
+    pub fn add_network<N>(&mut self, network: N) where N: Into<NetworkRef> {
+        self.add_nic(ServerNIC::FromNetwork(network.into()));
+    }
+
+    /// Add a virtual NIC to the new server.
+    pub fn add_nic(&mut self, nic: ServerNIC) {
+        self.networks.push(nic);
+    }
+
+    /// Add a virtual NIC with this port to the new server.
+    ///
+    /// A shorthand for `add_nic`.
+    pub fn add_port<P>(&mut self, port: P) where P: Into<PortRef> {
+        self.add_nic(ServerNIC::WithPort(port.into()));
+    }
+
+    /// Use this image as a source for the new server.
+    pub fn set_image<I>(&mut self, image: I) where I: Into<ImageRef> {
+        self.image = Some(image.into());
+    }
+
+    /// Add a virtual NIC with given fixed IP to the new server.
+    pub fn with_fixed_ip(mut self, fixed_ip: Ipv4Addr) -> NewServer<'session> {
+        self.add_fixed_ip(fixed_ip);
+        self
+    }
+
+    /// Use this image as a source for the new server.
+    pub fn with_image<I>(mut self, image: I) -> NewServer<'session>
+            where I: Into<ImageRef> {
+        self.set_image(image);
+        self
+    }
+
+    /// Add a virtual NIC from this network to the new server.
+    pub fn with_network<N>(mut self, network: N) -> NewServer<'session>
+            where N: Into<NetworkRef> {
+        self.add_network(network);
+        self
+    }
+
+    /// Add a virtual NIC with this port to the new server.
+    pub fn with_port<P>(mut self, port: P) -> NewServer<'session>
+            where P: Into<PortRef> {
+        self.add_port(port);
+        self
+    }
+}
+
+impl<'server> Waiter<Server<'server>, Error> for ServerCreationWaiter<'server> {
+    fn default_wait_timeout(&self) -> Option<Duration> {
+        Some(Duration::new(1800, 0))
+    }
+
+    fn default_delay(&self) -> Duration {
+        Duration::new(5, 0)
+    }
+
+    fn timeout_error(&self) -> Error {
+        Error::new(ErrorKind::OperationTimedOut,
+                   format!("Timeout waiting for server {} to become ACTIVE",
+                           self.server.id()))
+    }
+
+    fn poll(&mut self) -> Result<Option<Server<'server>>> {
+        self.server.refresh()?;
+        if self.server.status() == protocol::ServerStatus::Active {
+            debug!("Server {} successfully created", self.server.id());
+            // TODO(dtantsur): get rid of clone?
+            Ok(Some(self.server.clone()))
+        } else if self.server.status() == protocol::ServerStatus::Error {
+            debug!("Failed create server {} - status is ERROR",
+                   self.server.id());
+            Err(Error::new(ErrorKind::OperationFailed,
+                           format!("Server {} got into ERROR state",
+                                   self.server.id())))
+        } else {
+            trace!("Still waiting for server {} to become ACTIVE, current is {}",
+                   self.server.id(), self.server.status());
+            Ok(None)
+        }
+    }
+}
+
+impl<'server> WaiterCurrentState<Server<'server>> for ServerCreationWaiter<'server> {
+    fn waiter_current_state(&self) -> &Server<'server> {
+        &self.server
+    }
+}
+
 impl<'session> ResourceId for ServerSummary<'session> {
     fn resource_id(&self) -> String {
         self.id().clone()
