@@ -258,6 +258,18 @@ impl<'session> IntoFallibleIterator for ImageQuery<'session> {
 
 impl<'session> From<Image<'session>> for ImageRef {
     fn from(value: Image<'session>) -> ImageRef {
-        ImageRef::from(value.inner.id)
+        ImageRef::new_verified(value.inner.id)
+    }
+}
+
+impl ImageRef {
+    /// Verify this reference and convert to an ID, if possible.
+    #[cfg(feature = "image")]
+    pub(crate) fn into_verified(self, session: &Session) -> Result<String> {
+        Ok(if self.verified {
+            self.value
+        } else {
+            session.get_image(&self.value)?.id
+        })
     }
 }
