@@ -16,6 +16,7 @@
 
 use std::cell::Ref;
 
+use log;
 use reqwest::{Body, Method, RequestBuilder as ReqwestRB, Response, Url};
 use reqwest::header::{Header, Headers};
 use serde::Serialize;
@@ -116,9 +117,18 @@ impl RequestBuilder {
     }
 }
 
-#[inline]
-fn _log(resp: Response) -> Response {
-    trace!("HTTP request to {} returned {}", resp.url(), resp.status());
+fn _log(mut resp: Response) -> Response {
+    if log_enabled!(log::Level::Trace) {
+        let details = if resp.status().is_client_error() || resp.status().is_server_error() {
+            resp.text().ok()
+        } else {
+            None
+        };
+
+        // TODO(dtantsur): proper error parsing
+        trace!("HTTP request to {} returned {}; error: {:?}",
+               resp.url(), resp.status(), details);
+    }
     resp
 }
 
@@ -218,13 +228,26 @@ impl Session {
                                Some(self.endpoint_interface.clone()))
     }
 
-    fn get_service_info_ref<Srv>(&self)
+    pub(crate) fn get_service_info_ref<Srv>(&self)
             -> Result<Ref<ServiceInfo>> where Srv: ServiceType {
         self.ensure_service_info::<Srv>()?;
         Ok(self.cached_info.get_ref(&Srv::catalog_type()).unwrap())
     }
 }
 
+impl ServiceInfo {
+    /// Whether this service supports the given API version.
+    ///
+    /// Defaults to false if cannot be determined.
+    pub fn supports_api_version(&self, version: ApiVersion) -> bool {
+        match (self.minimum_version, self.current_version) {
+            (Some(min), Some(max)) => min <= version && max >= version,
+            (None, Some(current)) => current == version,
+            (Some(min), None) => version >= min,
+            _ => false
+        }
+    }
+}
 
 #[cfg(test)]
 mod test {
